// This file is part of OpenCV project.
// It is subject to the license terms in the LICENSE file found in the top-level directory
// of this distribution and at http://opencv.org/license.html.
//
// Copyright (C) 2019 Intel Corporation

#ifndef OPENCV_GAPI_RENDER_ROOT_HPP
#define OPENCV_GAPI_RENDER_ROOT_HPP

// This file is just a shortcut to render/render.hpp

#include <opencv2/gapi/render/render.hpp>

#endif // OPENCV_GAPI_RENDER_ROOT_HPP
